@@ -133,11 +133,9 @@ class Production(Workflow, ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Production, cls).__setup__()
-        cls._constraints += [
-            ('check_cost', 'missing_cost'),
-            ]
         cls._error_messages.update({
-                'missing_cost': 'It misses some cost on the outputs!',
+                'missing_cost': ('Production "%s" misses costs on '
+                    'some of its outputs.'),
                 })
         cls._transitions |= set((
                 ('request', 'draft'),
@@ -258,6 +256,7 @@ class Production(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Uom = pool.get('product.uom')
         Template = pool.get('product.template')
+        Product = pool.get('product.product')
         if not (self.bom and self.product and self.uom):
             return {}
         inputs = {
@@ -292,6 +291,10 @@ class Production(Workflow, ModelSQL, ModelView):
                 changes['cost'] += (Decimal(str(quantity)) *
                     input_.product.cost_price)
 
+        if hasattr(Product, 'cost_price'):
+            digits = Product.cost_price.digits
+        else:
+            digits = Template.cost_price.digits
         for output in self.bom.outputs:
             quantity = output.compute_quantity(factor)
             values = self._explode_move_values(self.location, storage_location,
@@ -299,10 +302,9 @@ class Production(Workflow, ModelSQL, ModelView):
             if values:
                 values['unit_price'] = Decimal(0)
                 if output.product.id == values.get('product') and quantity:
-                    digits = Template.cost_price.digits[1]
                     values['unit_price'] = Decimal(
                         changes['cost'] / Decimal(str(quantity))
-                        ).quantize(Decimal(str(10 ** -digits)))
+                        ).quantize(Decimal(str(10 ** -digits[1])))
                 outputs['add'].append(values)
         return changes
 
@@ -328,7 +330,8 @@ class Production(Workflow, ModelSQL, ModelView):
             result['uom.rec_name'] = ''
             result['unit_digits'] = 2
 
-        self.uom = result['uom']
+        if 'uom' in result:
+            self.uom = result['uom']
         if 'bom' in result:
             self.bom = result['bom']
         result.update(self.explode_bom())
@@ -381,6 +384,10 @@ class Production(Workflow, ModelSQL, ModelView):
         return cost
 
     def set_moves(self):
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+
         storage_location = self.warehouse.storage_location
         location = self.location
         company = self.company
@@ -393,6 +400,7 @@ class Production(Workflow, ModelSQL, ModelView):
                     move.production_output = self
                     move.unit_price = Decimal(0)
                     move.save()
+            self._set_move_planned_date()
             return
 
         factor = self.bom.compute_factor(self.product, self.quantity, self.uom)
@@ -407,6 +415,11 @@ class Production(Workflow, ModelSQL, ModelView):
                 move.save()
                 cost += Decimal(str(quantity)) * product.cost_price
 
+        if hasattr(Product, 'cost_price'):
+            digits = Product.cost_price.digits
+        else:
+            digits = Template.cost_price.digits
+
         for output in self.bom.outputs:
             quantity = output.compute_quantity(factor)
             product = output.product
@@ -415,32 +428,41 @@ class Production(Workflow, ModelSQL, ModelView):
             if move:
                 move.production_output = self
                 if product == self.product:
-                    move.unit_price = cost / Decimal(str(quantity))
+                    move.unit_price = Decimal(
+                        cost / Decimal(str(quantity))
+                        ).quantize(Decimal(str(10 ** -digits[1])))
                 move.save()
         self._set_move_planned_date()
 
+    @classmethod
+    def validate(cls, productions):
+        super(Production, cls).validate(productions)
+        for production in productions:
+            production.check_cost()
+
     def check_cost(self):
         if self.state != 'done':
-            return True
+            return
         cost_price = Decimal(0)
         for output in self.outputs:
             cost_price += (Decimal(str(output.quantity))
                 * output.unit_price)
         if not self.company.currency.is_zero(self.cost - cost_price):
-            return False
-        return True
+            self.raise_user_error('missing_cost', (self.rec_name,))
 
     @classmethod
-    def create(cls, values):
+    def create(cls, vlist):
         Sequence = Pool().get('ir.sequence')
         Config = Pool().get('production.configuration')
 
-        values = values.copy()
+        vlist = [x.copy() for x in vlist]
         config = Config(1)
-        values['code'] = Sequence.get_id(config.production_sequence.id)
-        production = super(Production, cls).create(values)
-        production._set_move_planned_date()
-        return production
+        for values in vlist:
+            values['code'] = Sequence.get_id(config.production_sequence.id)
+        productions = super(Production, cls).create(vlist)
+        for production in productions:
+            production._set_move_planned_date()
+        return productions
 
     @classmethod
     def write(cls, productions, values):
@@ -473,11 +495,8 @@ class Production(Workflow, ModelSQL, ModelView):
     def cancel(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
-        Move.write([m for p in productions
-                for m in p.inputs + p.outputs
-                if m.state != 'cancel'], {
-                'state': 'cancel',
-                })
+        Move.cancel([m for p in productions
+                for m in p.inputs + p.outputs])
 
     @classmethod
     @ModelView.button
@@ -485,11 +504,8 @@ class Production(Workflow, ModelSQL, ModelView):
     def draft(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
-        Move.write([m for p in productions
-                for m in p.inputs + p.outputs
-                if m.state != 'draft'], {
-                'state': 'draft',
-                })
+        Move.draft([m for p in productions
+                for m in p.inputs + p.outputs])
 
     @classmethod
     @ModelView.button
@@ -497,11 +513,8 @@ class Production(Workflow, ModelSQL, ModelView):
     def wait(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
-        Move.write([m for p in productions
-                for m in p.inputs + p.outputs
-                if m.state not in ('draft', 'done')], {
-                'state': 'draft',
-                })
+        Move.draft([m for p in productions
+                for m in p.inputs + p.outputs])
 
     @classmethod
     @Workflow.transition('assigned')
@@ -514,11 +527,7 @@ class Production(Workflow, ModelSQL, ModelView):
     def run(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
-        Move.write([m for p in productions
-                for m in p.inputs
-                if m.state not in ('done', 'cancel')], {
-                'state': 'done',
-                })
+        Move.do([m for p in productions for m in p.inputs])
 
     @classmethod
     @ModelView.button
@@ -527,11 +536,7 @@ class Production(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Move = pool.get('stock.move')
         Date = pool.get('ir.date')
-        Move.write([m for p in productions
-                for m in p.outputs
-                if m.state not in ('done', 'cancel')], {
-                'state': 'done',
-                })
+        Move.do([m for p in productions for m in p.outputs])
         cls.write(productions, {
                 'effective_date': Date.today(),
                 })
@@ -558,10 +563,7 @@ class Production(Workflow, ModelSQL, ModelView):
     def assign_force(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
-        Move.write([m for p in productions for m in p.inputs
-                if m.state != 'done'], {
-                'state': 'assigned',
-                })
+        Move.assign([m for p in productions for m in p.inputs])
         cls.assign(productions)
 
 
